package ru.ifmo.ga;

import java.util.Arrays;

public class SelectorFactory {
    private final static SelectorFactory instance = new SelectorFactory();

    public static SelectorFactory getInstance() {
        return instance;
    }

    public Selector randomSelector() {
        return new Selector() {
            public int select(double f[]) {
                return (int) (f.length * Math.random());
            }
        };
    }

    public Selector eliteSelector() {
        return new Selector() {
            private static final double ELITE_CUT = 0.25;

            public int select(double f[]) {
                return (int) (Math.random() * ELITE_CUT * f.length);
            }
        };
    }

    public Selector rouletteSelector() {
        return new Selector() {
            public int select(double f[]) {
                int i = Arrays.binarySearch(f, Math.random());
                return i >= 0 ? i : -(i + 1) - 1;
            }
        };
    }

    private class RankSelector implements Selector {
        private double x[];
        private final Selector sel = rouletteSelector();

        public int select(double f[]) {
            if (x == null) {
                init(f.length);
            }
            return sel.select(x);
        }

        private void init(int n) {
            x = new double[n];
            for (int i = 1; i < n; i++) {
                x[i] = x[i - 1] + n - (i - 1);
            }
        }
    }

    public Selector rankSelector() {
        return new RankSelector();
    }

}
