package ru.ifmo.robocode;

import ru.ifmo.robocode.gep.Chromosome;

public class AutomataFunction implements TankControlSystem {
    private static final int ACT = 4;
    private static final int N = 2;

    public static final int NUM = N * ACT + N * (N - 1);

    private final Expression[][] expr = new Expression[N][ACT];
    private final Expression[][] g = new Expression[N][N];

    private int state;

    public void newRound() {
        state = 0;
    }

    public int getState() {
        return state;
    }

    public double[] execute(double... s) {
        for (int i = 0; i < N; i++) {
            if (i != state) {
                double v = g[state][i].calc(s);
                if (v > 0) {
                    state = i;
                }
            }
        }

        double r[] = new double[ACT];
        for (int i = 0; i < r.length; i++) {
            r[i] = expr[state][i].calc(s);
        }

        return r;
    }

    public AutomataFunction(Chromosome c, BasisFunc[] basis, String[] terms) {
        int k = 0;

        for (int i = 0; i < N; i++) {
            for (int j = 0; j < ACT; j++) {
                expr[i][j] = new Expression(c.f[k++], basis, terms);
            }
        }

        for (int i = 0; i < N; i++) {
            for (int j = 0; j < N; j++) {
                if (i != j) {
                    g[i][j] = new Expression(c.f[k++], basis, terms);
                }
            }
        }

    }

}

