package ru.ifmo.robocode;

import ru.ifmo.robocode.BasisFunc;

public enum Basis implements BasisFunc {
    SIG(1, "s") {
        public double f(double... a) {
            return 1 / (1 + Math.exp(-a[0]));
        }

        public String toString(String... args) {
            return "s(" + args[0] + ")";
        }
    },

    NEG(1, "n"){
        public String toString(String... s) {
            return "-" + s[0];
        }

        public double f(double ...a) {
            return -a[0];
        }
    },

    PLUS(2, "+") {
        public double f(double ...a) {
            return a[0] + a[1];
        }

        public String toString(String... s) {
            return "(" + s[0] + "+" + s[1] + ")";
        }
    },

    PLUS_3(3, "+") {
        public double f(double ...a) {
            return a[0] + a[1] + a[2];
        }

        public String toString(String... s) {
            return "(" + s[0] + "+" + s[1] + "+" + s[2] + ")";
        }
    },

    MUL(2, "*"){
        public double f(double ...a) {
            return a[0] * a[1];
        }

        public String toString(String... s) {
            return "(" + s[0] + "*" + s[1] + ")";
        }
    },

    MUL_3(3, "*"){
        public double f(double ...a) {
            return a[0] * a[1] * a[2];
        }

        public String toString(String... s) {
            return "(" + s[0] + "*" + s[1] + "*" + s[2] + ")";
        }
    },

    MIN(2, "min"){
        public double f(double ...a) {
            return Math.min(a[0], a[1]);
        }

        public String toString(String... s) {
            return "min(" + s[0] + ", " + s[1] + ")";
        }
    },

    IFMORE(4, "if>") {
        public double f(double ...a) {
            return a[0] > a[1] ? a[2] : a[3];
        }

        public String toString(String... s) {
            return "(" + s[0] + ">" + s[1] + "?" + s[2] + ":" + s[3] + ")";
        }
    };

    public final int arity;
    public final String s;

    Basis(int arity, String s) {
        this.arity = arity;
        this.s = s;
    }

    public int arity() {
        return arity;
    }

    public String s() {
        return s;
    }
}
