package ru.ifmo.robocode;

import ru.ifmo.robocode.Environment;
import ru.ifmo.robocode.TankControlSystem;
import robocode.AdvancedRobot;
import robocode.HitByBulletEvent;
import robocode.ScannedRobotEvent;
import robocode.util.Utils;

public class SimpleTank extends AdvancedRobot {

    private static final double RADAR_ANG_STEP = Math.PI / 4;

    private long t, tHit;
    private double x, y, h, v, e;

    private TankControlSystem control;

    private Driver driver = new Driver();
    private Gun gun = new Gun();
    
    public void setModel(TankControlSystem control) {
        this.control = control;
    }

    public void run() {
        setAdjustRadarForGunTurn(true);
        x = y = h = v = e = 0;
        t = tHit = -1;
        control.newRound();
        while (true) {
            if (getRadarTurnRemainingRadians() == 0) {
                setTurnRadarRightRadians(RADAR_ANG_STEP);
            }
            if (t == -1) {      // enemy doesn't 'locked' yet
                doNothing();
                continue;
            }
            Environment env = new Environment(
                    x - getX(), y - getY(), v, h, getTime() - t, getX(), getY(), getVelocity(), getHeadingRadians(),
                    getGunHeadingRadians(), getEnergy(), e, getTime() - tHit, minWall(), getDistanceRemaining(), getTurnRemainingRadians()
            );
            double[] act = control.execute(env.values());
            gun.turn(this, act[0]);
            if (getGunHeat() == 0 && act[1] > 0) {
                gun.fire(this, act[1]);
            }
            driver.go(this, act[2]);
            driver.turn(this, act[3]);
            execute();
        }
    }

    public void onScannedRobot(ScannedRobotEvent event) {
        double d = event.getDistance();
        double a = Utils.normalAbsoluteAngle(getHeadingRadians() + event.getBearingRadians());
        x = d * Math.sin(a) + getX();
        y = d * Math.cos(a) + getY();
        t = event.getTime();
        v = event.getVelocity();
        h = event.getHeadingRadians();
        e = event.getEnergy();
    }

    public void onHitByBullet(HitByBulletEvent e) {
        tHit = e.getTime();
    }

    private double minWall() {
        return Math.min(Math.min(getX(), getBattleFieldWidth() - getX()), Math.min(getY(), getBattleFieldHeight() - getY()));
    }

    class Driver {
        public void go(AdvancedRobot tank, double dist) {
            tank.setAhead(dist);
        }

        public void turn(AdvancedRobot tank, double ang) {
            tank.setTurnRightRadians(ang);
        }
    }

    class Gun {
        public void fire(AdvancedRobot tank, double energy) {
            tank.setFire(energy);
        }

        public void turn(AdvancedRobot tank, double ang) {
            tank.setTurnGunRightRadians(ang);
        }
    }

}
