package ru.ifmo.robocode;

import newCynic.Cynical;
import robocode.Robot;

public class TankManager {
    private static TankManager ourInstance = new TankManager();

    private Robot tank;
    private final Class[] enemies = {Cynical.class};
    private final Robot[] robots = new Robot[enemies.length];

    private double [][][] pos;

    public double[] getPosition(String name, int roundNum) {
        int npos = roundNum / 2;
        int we = name.equals(tank.getClass().getName()) ? 1 : 0;
        int odd = roundNum % 2;
        return pos[npos][we ^ odd];
    }

    private boolean object = true;
    private boolean enemyObject = true;

    public void setObjects(boolean object, boolean enemyObject) {
        this.object = object;
        this.enemyObject = enemyObject;
    }

    public void setPositions(double[][][] positions) {
        pos = positions;
    }

    public boolean isPositions() {
        return pos != null;
    }

    public static TankManager getInstance() {
        return ourInstance;
    }

    private TankManager() {

    }

    public Robot getTank() {
        return tank;
    }


    public double getSpry() {
        return 1.0 - hit * 1.0 / fired;
    }

    public Robot getRobot(String clazz) throws IllegalAccessException, InstantiationException {
        if (object && clazz.equals(tank.getClass().getName())) {
            return tank;
        }
        if (!enemyObject) {
            return null;
        }
        for (int i = 0; i < enemies.length; i++) {
            Class c = enemies[i];
            if (c.getName().equals(clazz)) {
                return robots[i] = (Robot) c.newInstance();
            }
        }
        return null;
    }

    private int fired;
    private int hit;
    private double myEnergy;
    private double enemyEnergy;

    public double getEnemyEnergy() {
        return enemyEnergy;
    }

    public double getMyEnergy() {
        return myEnergy;
    }

    public void fire() {
        fired++;
    }

    public void hit() {
        hit++;
    }

    public void setTank(Robot tank) {
        this.tank = tank;
        hit = fired = 0;
        myEnergy = enemyEnergy = 0;
    }

    public Robot[] getRobots() {
        return robots;
    }

    public String[] getEnemies() {
        String names[] = new String[enemies.length];
        for (int i = 0; i < names.length; i++) {
            names[i] = enemies[i].getName();
        }
        return names;
    }

    public void addMyEnergy(double energy) {
        myEnergy += energy;
    }

    public void addEnemyEnergy(double energy) {
        enemyEnergy += energy;
    }

}
