package ru.ifmo.robocode.ga;

public abstract class Configuration<T> {

    abstract protected Selector createSelector();

    abstract protected Mutation<T> createMutation();

    abstract protected Crossover<T> createCrossover();

    abstract protected Randomizer<T> createRandomizer();

    abstract protected Utils<T> createUtils();

    protected int populationSize = 0;
    protected int chromosomeLength = 0;

    protected Fitness<T> fitness;
    protected Selector selector = createSelector();
    protected Mutation<T> mutation = createMutation();
    protected Crossover<T> crossover = createCrossover();
    protected Randomizer<T> randomizer = createRandomizer();
    protected Utils<T> utils = createUtils();

    public void setMutation(Mutation<T> mutation) {
        this.mutation = mutation;
    }

    public Crossover<T> getCrossover() {
        return crossover;
    }

    public Mutation<T> getMutation() {
        return mutation;
    }

    public Randomizer<T> getRandomizer() {
        return randomizer;
    }

    public Utils<T> getUtils() {
        return utils;
    }

    public Selector getSelector() {
        return selector;
    }

    public void setSelector(Selector selector) {
        this.selector = selector;
    }


    public void setFitness(Fitness<T> fitness) {
        this.fitness = fitness;
    }

    public void setRandomizer(Randomizer<T> randomizer) {
        this.randomizer = randomizer;
    }

    public Fitness<T> getFitness() {
        return fitness;
    }

    public int getChromosomeLength() {
        return chromosomeLength;
    }

    public void setChromosomeLength(int chromosomeLength) {
        this.chromosomeLength = chromosomeLength;
    }

    public int getPopulationSize() {
        return populationSize;
    }

    public void setPopulationSize(int populationSize) {
        this.populationSize = populationSize;
    }

    private void throwIllegal(String s) {
        throw new IllegalArgumentException(s);
    }

    public void checkInitialized() {
        if (populationSize <= 0) {
            throwIllegal("population size should be positive");
        }
        if (chromosomeLength <= 0) {
            throwIllegal("chromosome length should be positive");
        }
        if (fitness == null) {
            throwIllegal("fitness should be set");
        }
    }

}
