package ru.ifmo.robocode.ga;

public class SelectorFactory {
    private final static SelectorFactory instance = new SelectorFactory();

    public static SelectorFactory getInstance() {
        return instance;
    }

    public Selector randomSelector() {
        return new Selector() {
            public int select(double f[]) {
                return (int) (f.length * Math.random());
            }
        };
    }

    public Selector eliteSelector() {
        return new Selector() {
            private static final double ELITE_CUT = 0.25;

            public int select(double f[]) {
                return (int) (Math.random() * ELITE_CUT * f.length);
            }
        };
    }

    public Selector rouletteSelector() {
        return new Selector() {

            public int select(double f[]) {

//                double min = Long.MAX_VALUE;
//                for (double v : f) {
//                    min = Math.min(min, v);
//                }
//                if (min < 0) {
//                    for (int i = 0; i < f.length; i++) {
//                        f[i] += -min;
//                    }
//                }

                double sum = 0;
                for (double x : f) {
                    sum += x;
                }
                double x = Math.random() * sum;
                for (int i = 0; i < f.length - 1; i++) {
                    x -= f[i];
                    if (x < 0) {
                        return i;
                    }
                }
                return f.length - 1;
            }
        };
    }

    public Selector rankSelector() {
        return new Selector() {
            public int select(double f[]) {
                double r[] = new double[f.length];
                for (int i = 0; i < r.length; i++) {
                    r[i] = r.length - i;
                }
                int sum = (1 + r.length) * r.length / 2;
                double x = Math.random() * sum;
                for (int i = 0; i < r.length - 1; i++) {
                    x -= r[i];
                    if (x < 0) {
                        return i;
                    }
                }
                return f.length - 1;
            }
        };
    }
}
