package ru.ifmo.robocode.gep;

import ru.ifmo.robocode.BasisFunc;
import ru.ifmo.robocode.Expression;

import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

public class Chromosome {
    public final IntString f[];


    public void copy(Chromosome c) {
        for (int i = 0; i < f.length; i++) {
            f[i].copy(c.f[i]);
        }
    }

    public Chromosome(IntString f[]) {
        this.f = f;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();
        for (IntString e : f) {
            sb.append("\n").append(e.toString());
        }
        sb.deleteCharAt(0);
        return sb.toString();
    }

    public Chromosome(String s, BasisFunc[] basis, String terms[]) {
        String[] ss = s.split("\n");
        f = new IntString[ss.length];
        for (int i = 0; i < f.length; i++) {
            f[i] = fromCanonic(ss[i], basis, terms);
        }
    }

    private static class Term implements Comparable<Term> {
        private final String s;

        private final int h;
        private final int n;

        public Term(String s, int h, int n) {
            this.s = s;
            this.n = n;
            this.h = h;
        }

        public int compareTo(Term t) {
            int z = h - t.h;
            return z != 0 ? z : n - t.n;
        }
    }

    private IntString fromCanonic(String s, BasisFunc[] basis, String terms[]) {
        HashMap<String, Integer> map = getMap(basis, terms);
        ArrayList<Term> x = new ArrayList<Term>();
        goCanonic(s, x, 0);
        Collections.sort(x);
        int z[] = new int[x.size()];
        for (int i = 0; i < z.length; i++) {
            String term = x.get(i).s.trim();
            if (!map.containsKey(term)) {
                throw new RuntimeException("Unknown term: '" + term + "'");
            }
            z[i] = map.get(term);
        }
        return new IntString(z);
    }

    private HashMap<String, Integer> getMap(BasisFunc[] basis, String terms[]) {
        HashMap<String, Integer> map = new HashMap<String, Integer>();
        for (int i = 0; i < basis.length; i++) {
            map.put(basis[i].s(), i);
        }
        for (int i = basis.length; i < basis.length + terms.length; i++) {
            map.put(terms[i - basis.length], i);
        }
        return map;
    }

    private void goCanonic(String s, ArrayList<Term> x, int h) {
        int i = s.indexOf("(");
        if (i == -1) {
            x.add(new Term(s, h, x.size()));
            return;
        }
        String f = s.substring(0, i);
        x.add(new Term(f, h, x.size()));
        int j = s.lastIndexOf(")");
        String args = s.substring(i + 1, j);
        String a[] = splitArgs(args);
        for (String z : a) {
            goCanonic(z, x, h + 1);
        }
    }

    private String[] splitArgs(String s) {
        ArrayList<Integer> c = new ArrayList<Integer>();
        c.add(-1);
        int e = 0;
        for (int i = 0; i < s.length(); i++) {
            char ch = s.charAt(i);
            if (ch == '(') e++;
            else if (ch == ')') e--;
            else if (ch == ',' && e == 0) c.add(i);
        }
        c.add(s.length());
        String r[] = new String[c.size() - 1];
        for (int i = 0; i < r.length; i++) {
            r[i] = s.substring(c.get(i) + 1, c.get(i + 1));
        }
        return r;
    }

    public Chromosome(Reader r) throws IOException {
        BufferedReader br = new BufferedReader(r);
        int cnt = Integer.parseInt(br.readLine());
        f = new IntString[cnt];
        for (int i = 0; i < f.length; i++) {
            f[i] = new IntString(br.readLine());
        }
        br.close();
    }

    public void save(Writer w) {
        PrintWriter pw = new PrintWriter(w);
        pw.println(f.length);
        pw.println(toString());
        pw.close();
    }

    public String toCanonic(BasisFunc[] basis, String terms[]) {
        StringBuilder sb = new StringBuilder();
        for (IntString e : f) {
            sb.append("\n").append(new Expression(e, basis, terms).toCanonic());
        }
        sb.deleteCharAt(0);
        return sb.toString();
    }


}
