package ru.ifmo.robocode.ga;

import org.apache.log4j.Logger;

import java.util.Arrays;

public class Genome<T> {
    private final static Logger LOG = Logger.getLogger(Genome.class);


    private Configuration<T> cfg;

    private EvalChromosome<T>[] pop;
    private EvalChromosome<T>[] next;
    private double f[];

    private int generation;

    private double avg10() {
        int n = Math.min(10, pop.length);
        double res = 0;
        for (int i = 0; i < n; i++) {
            res += pop[i].v;
        }
        return res / n;
    }

    private double div10() {
        int n = Math.min(10, pop.length);
        double r = 0;
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < i; j++) {
                r += cfg.getUtils().diff(pop[i].c, pop[j].c);
            }
        }
        return r * 2.0 / (n * (n - 1));
    }

    private String getStat(long dt, double oldDiv) {
        return "max: " + pop[0].v + ", avg: " + String.format("%.2f", avg10()) + ", div: " + String.format("%.3f", div10()) /*+ " / " + String.format("%.3f", oldDiv)*/ + " in " + dt + " sec.";
    }

    private Genome(Configuration<T> configuration) {
        long time = System.currentTimeMillis();
        cfg = configuration;
        final int size = cfg.getPopulationSize();
        pop = new EvalChromosome[size];
        next = new EvalChromosome[size];
        f = new double[size];
        for (int i = 0; i < size; i++) {
            pop[i] = new EvalChromosome<T>(cfg.getRandomizer().randomize(cfg.getChromosomeLength()));
            next[i] = new EvalChromosome<T>(cfg.getUtils().allocate(cfg.getChromosomeLength()));
        }
        double oldDiv = div10();
        calcFitness();
        LOG.info("[created] " + getStat((System.currentTimeMillis() - time) / 1000, oldDiv));
    }

    private void calcFitness() {
        cfg.getFitness().fitness(pop);
        Arrays.sort(pop);
    }

    public void evolve() {
        long time = System.currentTimeMillis();
        final int size = cfg.getPopulationSize();
        for (int i = 0; i < size; i++) {
            f[i] = pop[i].v;
        }
        cfg.getUtils().copy(pop[0].c, next[0].c); // keep best one
        for (int i = 1; i < size; i++) {
            int j = cfg.getSelector().select(f);
            int k = cfg.getSelector().select(f);
            cfg.getCrossover().crossover(pop[j].c, pop[k].c, next[i].c);
            cfg.getMutation().mutate(next[i].c);
        }
        for (int i = 0; i < size; i++) {
            cfg.getUtils().copy(next[i].c, pop[i].c);
            pop[i].v = next[i].v;
        }
        double div = div10();
        calcFitness();
        generation++;
        LOG.info("[evolved] " + generation + ". " + getStat((System.currentTimeMillis() - time) / 1000, div));
    }

    public double getFittest() {
        return pop[0].v;
    }

    public T getFittestChromosome() {
        return pop[0].c;
    }

    public static <T> Genome<T> randomize(Configuration<T> cfg) {
        cfg.checkInitialized();
        return new Genome<T>(cfg);
    }

}
